module Saper
  module DSL
    
    # Returns a new empty module with embedded parsing methods.
    # @return [Object]
    def self.new
      Module.new.extend(Methods)
    end
    
    # Enables recipe parsing in modules that include Saper::DSL.
    # @return [void]
    def self.included(base)
      base.extend(Methods)
    end
    
    # A mixin module with a set of parsing methods
    module Methods
      
      # @todo
      def namespace
        @namespace ||= Saper::Namespace.new
      end
      
      # @todo
      def recipe(id, name = nil, &block)
        namespace[id] = Recipe.parse(id, name, namespace, &block)
      end
      
      # @todo
      def [](name)
        namespace[name]
      end
      
    end
    
    class Recipe
      # Parses block and returns a Recipe instance or a proxy object.
      # If namespace is specified within options, full initialization of
      # Recipe is delayed and a proxy object is returned (which supports
      # `#to_recipe`).
      # @param id [Symbol] recipe ID
      # @return [Saper::Recipe, Saper::DSL::Recipe]
      def self.parse(id = nil, name = nil, ns = nil, &block)
        instance = self.new(id, name, ns, &block)
        if ns.is_a?(Namespace)
          instance
        else
          instance.to_recipe
        end
      end
      
      attr_reader :recipe
      
      # @todo
      def initialize(id, name = nil, ns = nil, &block)
        @recipe, @ns, @block = Saper::Recipe.new(id, :name => name), ns, block
      end
      
      # @todo
      def to_recipe(*args)
        self.instance_eval(&@block) if recipe.empty?; recipe
      end
      
      # @todo
      def method_missing(name, *args, &block)
        @recipe << Saper::Action.new(name, *args, :namespace => @ns, &block)
      end
      
    end
  end
end
