module Saper
  
  # Generic exception acting as a parent class for all exceptions in Saper library.
  class Error < StandardError; end
  
  # FileUnreadable is raised whenever a recipe file cannot be read.
  class FileUnreadable < Error; end
  
  # NoAction is raised whenever an implementation of Saper::Action is not found.
  class ActionNotFound < Error; end
  
  # NamespaceMissing is raised whenever Saper::Action requires a missing namespace.
  class NamespaceMissing < Error; end
  
  # NoAction is raised whenever an unsupported object is added to Saper::Recipe.
  class ActionExpected < Error; end
  
  # InvalidAction is raised whenever serialized representation of Saper::Action is invalid.
  class InvalidAction < Error; end
  
  # InvalidAction is raised whenever serialized representation of Saper::Recipe is invalid.
  class InvalidRecipe < Error; end

  # InvalidNamespace is raised whenever serialized representation of Saper::Namespace is invalid.
  class InvalidNamespace < Error; end
  
  # InvalidType is raised whenever argument type is not recognized.
  class InvalidType < Error; end
  
  # InvalidArgument is raised whenever an argument has incorrect type or value.
  class InvalidArgument < Error; end
  
  # InvalidInput is raised whenever Saper::Action does not support this type of input.
  class InvalidInput < Error; end
  
  # InvalidOutput is raised when execution of an action yields unexpected results.
  class InvalidOutput < Error; end
  
  # RecipeNotFound is raised whenever action calls a recipe that cannot be found.
  class RecipeNotFound < Error; end
  
  # UnreachableURL is raised whenever a connection error occurs.
  class UnreachableURL < Error; end
  
  # InvalidItem is raised whenever Saper::Item is initialized with an invalid underlying.
  class InvalidItem < Error; end
  
  # @todo
  class RuntimeMissing < Error; end
end