module Saper
  class Stack
    
    # Returns Runtime instance used by the stack
    # @return [Saper::Runtime]
    attr_reader :runtime
    
    # Returns all items in the stack.
    # @return [Array<Saper::Item>]
    attr_reader :items
    
    # Returns a new Stack instance.
    # @param runtime [Saper::Runtime]
    # @param items [Saper::Item]
    # @return [Saper::Stack]
    def initialize(runtime, *items)
      @runtime  = runtime
      @items    = items.flatten
    end
    
    # @todo
    def size
      @items.size
    end
    
    # Returns a duplicate instance of self.
    # @return [Saper::Stack]
    def copy
      Stack.new(runtime.copy, *to_a)
    end
    
    # Adds action result to the end of the stack and returns self.
    # @param item [Saper::Item]
    # @return [self]
    def add(item)
      @items.push(item); self
    end
    
    # Returns the result of the last action or nil (in case of an error).
    # @return [Saper::Item]
    def result
      error? ? Items::Nothing.new : last
    end
    
    # @todo
    def error
      error? ? last : nil
    end
    
    # Returns the result of the last action.
    # @return [Saper::Item]
    def last
      @items.last || Items::Nothing.new
    end
    
    # @todo
    def filtered?
      last.is_a?(Items::Nothing) && last.filtered?
    end
    
    # @todo
    def error?
      last.is_a?(Saper::Error)
    end
    
    # Returns an array of action results.
    # @return [Array<Saper::Item>]
    def to_a
      @items.dup
    end
  
  end
end