module Saper
  class Type
    
    # Returns a new Type instance of a given type.
    # @param type [Symbol]
    # @return [Saper::Type]
    def self.[](type)
      case type
        when Type       then type
        when :attribute then AttributeType.new
        when :document  then DocumentType.new
        when :html      then HTMLType.new
        when :json      then JSONType.new
        when :options   then OptionsType.new
        when :regex     then RegexType.new
        when :recipe    then RecipeType.new
        when :tag       then TagType.new
        when :text      then TextType.new
        when :url       then URLType.new
        when :variable  then VariableType.new
        when :xml       then XMLType.new
        when :xpath     then XPathType.new
        else raise(InvalidType, "Invalid action argument: %s" % type)
      end
    end
    
    # Returns normalize value
    def normalize(value)
      value # Note: subclass may override this method.
    end
    
    # Returns `true` if specified value matches type.
    # @return [Boolean]
    def valid?(value)
      true  # Note: subclass must override this method.
    end
    
    # Returns `true` if specified value does not match type.
    # @return [Boolean]
    def invalid?(value)
      not valid?(value)
    end
    
  end
end