module Saper
  module Items
    class HTML < Item
      
      # @todo
      def self.new(item)
        super case item
        when Nokogiri::XML::Element
          item
        when Nokogiri::XML::Document
          item
        when Nokogiri::HTML
          item
        when Document
          parse(item.body, item.uri, item.charset)
        when Text
          parse(item.to_s)
        when String
          parse(item)
        else
          raise(InvalidItem, item)
        end
      end
      
      # @todo
      def self.parse(string, uri = nil, charset = nil)
        if string.empty?
          raise(InvalidItem, string)
        end
        begin
          # Nokogiri adds an additional body tag for strings that don't
          # have one. Should probably use Nokogiri::HTML.fragment in that
          # case, but it breaks the search function:
          # https://github.com/sparklemotion/nokogiri/issues/572
          Nokogiri::HTML.parse(string, uri, charset)
        rescue
          raise(InvalidItem, string)
        end
      end
      
      # @todo
      def initialize(noko)
        @noko = noko
        # Force UTF-8 encoding
        # https://github.com/sparklemotion/nokogiri/issues/117
        @noko.document.encoding = 'UTF-8'
      end
      
      # @todo
      def dup
        HTML.new(@noko.dup)
      end
      
      # @todo
      def ==(other)
        to_s == other.to_s
      end
      
      # @todo
      def name
        @noko.name
      end
      
      # @todo
      def [](name)
        @noko.has_attribute?(name) ? Text.new(@noko[name]) : nil
      end
      
      # @todo
      def find(xpath)
        find_all(xpath).first
      end
      
      # @todo
      def find_all(xpath)
        @noko.search(xpath).map { |element| HTML.new(element) }
      end
      
      # @todo
      def remove_children_with_content!(xpath)
        @noko.search(xpath).each { |item| item.remove }; self
      end
      
      # @todo
      def remove_children_with_content(xpath)
        dup.remove_children_with_content!(xpath)
      end
      
      # @todo
      def remove_children_preserving_content!(xpath)
        @noko.search(xpath).each { |item| item.replace(item.children) }; self
      end

      # @todo
      def remove_children_preserving_content(xpath)
        dup.remove_children_preserving_content!(xpath)
      end
      
      # @todo
      def inner_html
        Text.new @noko.inner_html.gsub("\n","")
      end
      
      # @todo
      def inner_text
        Text.new @noko.inner_text
      end
      
      # @todo
      def to_markdown
        Markdown.new self
      end
      
      # @todo
      def to_native
        inner_html.to_native
      end
      
      # @todo
      def to_s
        @noko.is_a?(Nokogiri::HTML::Document) ? to_native : @noko.to_s.strip
      end
      
    end
  end
end