module Tungsten
  class Request
    
    attr_reader :method
    
    attr_reader :data
    
    attr_reader :jar
    
    def initialize(method, uri, data: nil, jar: nil, agent: :tungsten)
      @method = method.to_s.upcase
      @uri    = URI.parse(uri)
      @jar    = jar || Jar.new
      @agent  = agent
      @data   = data.to_s
      @data   = data.read if data.is_a?(IO)
      @req    = net_http_request
    end
    
    def response
      http = Net::HTTP.new(host, port)
      http.use_ssl = true if ssl?
      resp = http.request(@req)
      jar.add_from_response(resp)
      Response.new(@uri, resp)
    rescue => exception
      puts 'ERROR: %s' % exception
      nil
    end
    
    def headers
      Headers.new(@req.to_hash)
    end
    
    def scheme
      @uri.scheme
    end
    
    def port
      @uri.port
    end
    
    def host
      @uri.host
    end
    
    def path
      @uri.path.empty? ? "/" : @uri.path
    end
    
    def query
      @uri.query
    end
    
    def fragment
      @uri.fragment
    end
    
    def uri
      @uri.to_s
    end
    
    def size
      headers.size + data.size
    end
    
    def user_agent
      AGENTS[@agent.to_sym] || @agent.to_s
    end
    
    def has_data?
      not @data.empty?
    end
    
    def ssl?
      port == 443
    end
    
    private
    
    def cookie_header
      jar.any?(uri) ? { "Cookie" => jar.for(@uri) } : {}
    end
    
    def custom_headers
      cookie_header.merge({ "User-Agent" => user_agent })
    end
    
    def req_body?
      %w(GET POST PUT DELETE TRACE PATCH PROPFIND PROPPATCH MKCOL COPY MOVE LOCK UNLOCK).include?(method)
    end
    
    def resp_body?
      %w(POST PUT PATCH PROPFIND PROPPATCH MKCOL LOCK UNLOCK).include?(method)
    end
    
    def net_http_request
      req = Net::HTTPGenericRequest.new(method, req_body?, resp_body?, path, custom_headers); req.body = data if has_data?; req
    end
    
  end
end