module Tungsten
  class Response
    
    def initialize(uri, http)
      @uri, @http = uri, http
    end
    
    def headers
      Headers.new(@http.to_hash)
    end
    
    def uri
      @uri.to_s
    end
    
    def code
      @http.code.to_i
    end
    
    def message
      @http.message
    end
    
    def body
      @http.body
    end
    
    def size
      headers.to_s.size + body.to_s.size
    end
    
    def ok?
      code == 200
    end
    
    def created?
      code == 201
    end
    
    def accepted?
      code == 202
    end
    
    def no_content?
      code == 204
    end
    
    def moved_permanently?
      code == 301
    end
    
    def moved_temporarily?
      code == 302 || code == 307
    end
    
    def see_other?
      code == 303
    end
    
    def not_modified?
      code == 304
    end
    
    def redirect?
      moved_permanently? || moved_temporarily? || see_other?
    end
    
    def bad_request?
      code == 400
    end
    
    def unauthorized?
      code == 401
    end
    
    def forbidden?
      code == 403
    end
    
    def not_found?
      code == 404
    end
    
    def method_not_allowed?
      code == 406
    end
    
    def client_error?
      code > 399 && code < 500
    end
    
    def server_error?
      code > 499
    end
    
    def error?
      client_error? || server_error?
    end
    
  end
end